# Copyright 2015  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# =*= License: GPL-3+ =*=


import shutil
import tempfile
import unittest

import icklib


class StateDirectoryTests(unittest.TestCase):

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def test_sets_dirname(self):
        statedir = icklib.StateDirectory()
        statedir.set_dirname('foo')
        self.assertEqual(statedir.dirname, 'foo')

    def test_returns_project_state_path(self):
        statedir = icklib.StateDirectory()
        statedir.set_dirname('foo')
        project_path = statedir.get_project_state_path('bar')
        self.assertEqual(project_path, 'foo/bar/project.yaml')

    def test_returns_git_directory_path(self):
        statedir = icklib.StateDirectory()
        statedir.set_dirname('foo')
        gitpath = statedir.get_git_directory('bar')
        self.assertEqual(gitpath, 'foo/bar/git')

    def test_returns_no_build_infos_initially(self):
        statedir = icklib.StateDirectory()
        statedir.set_dirname(self.tempdir)
        build_infos = statedir.get_build_infos('bar')
        self.assertEqual(build_infos, [])

    def test_creates_new_build(self):
        statedir = icklib.StateDirectory()
        statedir.set_dirname(self.tempdir)
        new_build = statedir.create_new_build('bar', 1)
        self.assertEqual(new_build.build_number, 1)

        build_infos = statedir.get_build_infos('bar')
        self.assertEqual(len(build_infos), 1)
        self.assertEqual(build_infos[0].build_number, new_build.build_number)

    def test_creates_two_builds(self):
        statedir = icklib.StateDirectory()
        statedir.set_dirname(self.tempdir)

        build_1 = statedir.create_new_build('bar', 1)
        build_2 = statedir.create_new_build('bar', 2)

        build_infos = statedir.get_build_infos('bar')
        self.assertEqual(len(build_infos), 2)
        self.assertEqual(build_infos[0].build_number, build_1.build_number)
        self.assertEqual(build_infos[1].build_number, build_2.build_number)

    def test_updates_build_info(self):
        statedir = icklib.StateDirectory()
        statedir.set_dirname(self.tempdir)

        build_info = statedir.create_new_build('bar', 1)
        build_info.commits = [
            {
                'git': 'git://git.example.com/foo.git',
                'commit': 'cafef00d',
            },
        ]
        statedir.update_build_info('bar', build_info)

        build_infos = statedir.get_build_infos('bar')
        self.assertEqual(len(build_infos), 1)
        self.assertEqual(build_infos[0].build_number, build_info.build_number)
        self.assertEqual(build_infos[0].commits, build_info.commits)
